//
//  ABFilterAppDelegate.m
//  Audiobus Samples
//
//  Created by Michael Tyson on 18/12/2013.
//  Copyright (c) 2011-2014 Audiobus. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
//
//  - Redistributions in binary form must reproduce the above copyright notice, this
//    list of conditions and the following disclaimer in the documentation and/or
//    other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "ABFilterAppDelegate.h"
#import "ABFilterViewController.h"
#import "Audiobus.h"
#import "ABFilterAudioEngine.h"

@interface ABFilterAppDelegate ()  <UIApplicationDelegate, ABAudiobusControllerStateIODelegate>
@property (strong, nonatomic) ABFilterViewController *viewController;
@property (strong, nonatomic) ABFilterAudioEngine *audioEngine;
@property (strong, nonatomic) ABFilterPort *filter;
@end

@implementation ABFilterAppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    
    application.statusBarHidden = YES;
    
    self.audioEngine = [ABFilterAudioEngine new];
    
    // Start audio engine in next event loop, so we have time to test the launch URL, below
    [_audioEngine performSelector:@selector(start) withObject:nil afterDelay:0.0];
    
    _audioEngine.audiobusController.stateIODelegate = self;
    _audioEngine.audiobusController.connectionPanelPosition = ABConnectionPanelPositionBottom;
    
    NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
    [ud registerDefaults:@{ @"Wet/Dry": @(50),
                            @"Delay": @(1),
                            @"Feedback": @(50),
                            @"Lopass": @(15000) }];
    
    _audioEngine.wetDryMix = [ud doubleForKey:@"Wet/Dry"];
    _audioEngine.delayTime = [ud doubleForKey:@"Delay"];
    _audioEngine.feedback = [ud doubleForKey:@"Feedback"];
    _audioEngine.lopassCutoff = [ud doubleForKey:@"Lopass"];
    
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.viewController = [[ABFilterViewController alloc] initWithAudioEngine:_audioEngine];
    self.window.rootViewController = self.viewController;
    [self.window makeKeyAndVisible];
    
    return YES;
}


-(NSDictionary *)audiobusStateDictionaryForCurrentState {
    // This method is called by Audiobus when the user saves a preset. This lets the app set some state to be
    // restored when the preset is loaded again.
    return @{ @"Wet/Dry": @(_audioEngine.wetDryMix),
              @"Delay": @(_audioEngine.delayTime),
              @"Feedback": @(_audioEngine.feedback),
              @"Lopass": @(_audioEngine.lopassCutoff)};
}

-(void)loadStateFromAudiobusStateDictionary:(NSDictionary *)dictionary responseMessage:(NSString *__autoreleasing *)outResponseMessage {
    // This will be called when the user loads a preset in Audiobus. Passed in is the same dictionary that was
    // returned above when the preset was saved, so we can restore state from it.
    _audioEngine.wetDryMix = [dictionary[@"Wet/Dry"] doubleValue];
    _audioEngine.delayTime = [dictionary[@"Delay"] doubleValue];
    _audioEngine.feedback = [dictionary[@"Feedback"] doubleValue];
    _audioEngine.lopassCutoff = [dictionary[@"Lopass"] doubleValue];
}

@end
