//
//  ABFilterViewController.m
//  Audiobus Samples
//
//  Created by Michael Tyson on 18/12/2013.
//  Copyright (c) 2011-2014 Audiobus. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
//
//  - Redistributions in binary form must reproduce the above copyright notice, this
//    list of conditions and the following disclaimer in the documentation and/or
//    other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "ABFilterViewController.h"
#import "ABFilterAudioEngine.h"
#import "TPOscilloscopeView.h"

@interface ABFilterViewController ()
@property (nonatomic, strong) TPOscilloscopeView *oscilloscope;
@end

static void * kWetDryChanged = &kWetDryChanged;
static void * kDelayTimeChanged = &kDelayTimeChanged;
static void * kFeedbackChanged = &kFeedbackChanged;
static void * kLopassChanged = &kLopassChanged;

@implementation ABFilterViewController

- (id)initWithAudioEngine:(ABFilterAudioEngine *)audioEngine  {
    if ( !(self = [super init]) ) return nil;
    self.audioEngine = audioEngine;
    return self;
}

-(void)dealloc {
    self.audioEngine = nil;
}

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    if ( [self.view respondsToSelector:@selector(tintColor)] ) {
        self.view.tintColor = [UIColor orangeColor];
    }
    
    _wetDrySlider.value = _audioEngine.wetDryMix;
    _delayTimeSlider.value = _audioEngine.delayTime;
    _feedbackSlider.value = _audioEngine.feedback;
    _lopassSlider.value = _audioEngine.lopassCutoff;
}

- (IBAction)setWetDry:(UISlider*)sender {
    _audioEngine.wetDryMix = sender.value;
}
- (IBAction)setDelayTime:(UISlider*)sender {
    _audioEngine.delayTime = sender.value;
}
- (IBAction)setFeedback:(UISlider*)sender {
    _audioEngine.feedback = sender.value;
}
- (IBAction)setLopass:(UISlider*)sender {
    _audioEngine.lopassCutoff = sender.value;
}

-(void)setAudioEngine:(ABFilterAudioEngine *)audioEngine {
    if ( _audioEngine ) {
        [_audioEngine removeObserver:self forKeyPath:@"wetDryMix"];
        [_audioEngine removeObserver:self forKeyPath:@"delayTime"];
        [_audioEngine removeObserver:self forKeyPath:@"feedback"];
        [_audioEngine removeObserver:self forKeyPath:@"lopassCutoff"];
    }
    
    _audioEngine = audioEngine;
    
    if ( _audioEngine ) {
        [_audioEngine addObserver:self forKeyPath:@"wetDryMix" options:0 context:kWetDryChanged];
        [_audioEngine addObserver:self forKeyPath:@"delayTime" options:0 context:kDelayTimeChanged];
        [_audioEngine addObserver:self forKeyPath:@"feedback" options:0 context:kFeedbackChanged];
        [_audioEngine addObserver:self forKeyPath:@"lopassCutoff" options:0 context:kLopassChanged];
    }
}

-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    if ( context == kWetDryChanged ) {
        _wetDrySlider.value = _audioEngine.wetDryMix;
        [[NSUserDefaults standardUserDefaults] setDouble:_audioEngine.wetDryMix forKey:@"Wet/Dry"];
    } else if ( context == kDelayTimeChanged ) {
        _delayTimeSlider.value = _audioEngine.delayTime;
        [[NSUserDefaults standardUserDefaults] setDouble:_audioEngine.delayTime forKey:@"Delay"];
    } else if ( context == kFeedbackChanged ) {
        _feedbackSlider.value = _audioEngine.feedback;
        [[NSUserDefaults standardUserDefaults] setDouble:_audioEngine.feedback forKey:@"Feedback"];
    } else if ( context == kLopassChanged ) {
        _lopassSlider.value = _audioEngine.lopassCutoff;
        [[NSUserDefaults standardUserDefaults] setDouble:_audioEngine.lopassCutoff forKey:@"Lopass"];
    } else {
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}

@end
