//
//  ABMultitrackPortView.m
//  Audiobus Samples
//
//  Created by Michael Tyson on 19/12/2013.
//  Copyright (c) 2011-2014 Audiobus. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
//
//  - Redistributions in binary form must reproduce the above copyright notice, this
//    list of conditions and the following disclaimer in the documentation and/or
//    other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "ABMultitrackPortView.h"
#import "TPOscilloscopeView.h"
#import "ABMultitrackAudioEngine.h"
#import "Audiobus.h"
#import <QuartzCore/QuartzCore.h>

static void * kPeerIconChanged = &kPeerIconChanged;

@interface ABMultitrackPortView ()
@property (nonatomic, strong) UIImageView *iconView;
@property (nonatomic, strong) TPOscilloscopeView *oscilloscopeView;
@end

@implementation ABMultitrackPortView

-(id)initWithPort:(ABPort *)port audioEngine:(ABMultitrackAudioEngine *)audioEngine frame:(CGRect)frame {
    if ( !(self = [super initWithFrame:CGRectZero]) ) return nil;
    
    self.port = port;
    self.audioEngine = audioEngine;
    
    self.iconView = [[UIImageView alloc] initWithFrame:CGRectZero];
    _iconView.translatesAutoresizingMaskIntoConstraints = NO;
    _iconView.layer.masksToBounds = YES;
    [self addSubview:_iconView];

    self.oscilloscopeView = [[TPOscilloscopeView alloc] initWithFrame:CGRectZero];
    _oscilloscopeView.translatesAutoresizingMaskIntoConstraints = NO;
    _oscilloscopeView.lineColor = [UIColor whiteColor];
    [self addSubview:_oscilloscopeView];
    
    [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"H:|-10-[_iconView]-10-[_oscilloscopeView]|" options:0 metrics:nil views:NSDictionaryOfVariableBindings(_iconView, _oscilloscopeView)]];
    [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-10-[_iconView]-10-|" options:0 metrics:nil views:NSDictionaryOfVariableBindings(_iconView)]];
    [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-10-[_oscilloscopeView]-10-|" options:0 metrics:nil views:NSDictionaryOfVariableBindings(_oscilloscopeView)]];
    [self addConstraint:[NSLayoutConstraint constraintWithItem:_iconView attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationEqual toItem:_iconView attribute:NSLayoutAttributeHeight multiplier:1 constant:0]];
    
    _iconView.image = port.peer.icon;
    
    __unsafe_unretained ABMultitrackPortView *weakSelf = self;
    [_audioEngine addAudioReceiveBlock:^(AudioBufferList *audio, UInt32 length, const AudioTimeStamp *timestamp) {
        [weakSelf->_oscilloscopeView addAudio:audio length:length];
    } forPort:_port withIdentifier:@((long)self)];
    
    [self setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisVertical];
    
    return self;
}

-(CGSize)intrinsicContentSize {
    BOOL isiPad = [[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad;
    return CGSizeMake(UIViewNoIntrinsicMetric, isiPad ? 150 : 100);
}

-(void)dealloc {
    self.port = nil;
    [_audioEngine removeAudioReceiveBlockWithIdentifier:@((long)self)];
}

-(void)setPort:(ABPort *)port {
    if ( _port ) {
        [_port.peer removeObserver:self forKeyPath:@"icon"];
    }
    
    _port = port;
    
    if ( _port ) {
        [_port.peer addObserver:self forKeyPath:@"icon" options:0 context:kPeerIconChanged];
    }
}

-(void)didMoveToSuperview {
    if ( self.superview ) {
        [_oscilloscopeView start];
    } else {
        [_oscilloscopeView stop];
    }
}

-(void)layoutSubviews {
    [super layoutSubviews];
    _iconView.layer.cornerRadius = (10.0/57.0) * _iconView.frame.size.width;
}

-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    if ( context == kPeerIconChanged ) {
        _iconView.image = _port.peer.icon;
    } else {
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}

@end
