//
//  ABMultitrackViewController.m
//  Audiobus Samples
//
//  Created by Michael Tyson on 18/12/2013.
//  Copyright (c) 2011-2014 Audiobus. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
//
//  - Redistributions in binary form must reproduce the above copyright notice, this
//    list of conditions and the following disclaimer in the documentation and/or
//    other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "ABMultitrackViewController.h"
#import "Audiobus.h"
#import "ABMultitrackAudioEngine.h"
#import "ABMultitrackPortView.h"

@interface ABMultitrackViewController ()
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) NSMutableDictionary *portViews;
@property (nonatomic, strong) NSMutableArray *customConstraints;
@property (nonatomic, strong) UILabel *messageLabel;
@end

@implementation ABMultitrackViewController

-(id)initWithAudioEngine:(ABMultitrackAudioEngine *)audioEngine {
    if ( !(self = [super init]) ) return nil;
    self.audioEngine = audioEngine;
    self.portViews = [NSMutableDictionary new];
    self.customConstraints = [NSMutableArray new];
    return self;
}

-(void)loadView {
    self.view = [[UIView alloc] initWithFrame:CGRectZero];
    self.view.backgroundColor = [UIColor colorWithRed:0.722 green:0.196 blue:0.204 alpha:1.000];
    
    self.scrollView = [[UIScrollView alloc] initWithFrame:CGRectZero];
    _scrollView.translatesAutoresizingMaskIntoConstraints = NO;
    [self.view addSubview:_scrollView];
    
    [self.view addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"H:|[_scrollView]|" options:0 metrics:nil views:NSDictionaryOfVariableBindings(_scrollView)]];
    [self.view addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|[_scrollView]|" options:0 metrics:nil views:NSDictionaryOfVariableBindings(_scrollView)]];
}

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateView) name:ABMultitrackAudioEnginePortAddedNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateView) name:ABMultitrackAudioEnginePortRemovedNotification object:nil];
    [self updateView];
}

-(void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

-(void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)updateView {
    [self.view removeConstraints:_customConstraints];
    [self.customConstraints removeAllObjects];
    
    NSArray *ports = _audioEngine.connectedPorts;
    
    ABMultitrackPortView *lastView = nil;
    for ( ABPort *port in ports ) {
        ABMultitrackPortView *view = _portViews[@((long)port)];
        if ( !view ) {
            view = [[ABMultitrackPortView alloc] initWithPort:port audioEngine:_audioEngine frame:CGRectZero];
            view.translatesAutoresizingMaskIntoConstraints = NO;
            [_portViews setObject:view forKey:@((long)port)];
            [_scrollView addSubview:view];
        }
        
        [_customConstraints addObject:[NSLayoutConstraint constraintWithItem:view attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeLeft multiplier:1 constant:5]];
        [_customConstraints addObject:[NSLayoutConstraint constraintWithItem:view attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeRight multiplier:1 constant:-5]];
        
        if ( lastView ) {
            [_customConstraints addObject:[NSLayoutConstraint constraintWithItem:view attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:lastView attribute:NSLayoutAttributeBottom multiplier:1 constant:5]];
        } else {
            [_customConstraints addObject:[NSLayoutConstraint constraintWithItem:view attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:_scrollView attribute:NSLayoutAttributeTop multiplier:1 constant:25]];
        }
        
        lastView = view;
    }
    
    if ( lastView ) {
        [_customConstraints addObject:[NSLayoutConstraint constraintWithItem:lastView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:_scrollView attribute:NSLayoutAttributeBottom multiplier:1 constant:-5]];
    }
    
    for ( ABMultitrackPortView *view in _portViews.allValues ) {
        if ( ![ports containsObject:view.port] ) {
            [_portViews removeObjectForKey:@((long)view.port)];
            [view removeFromSuperview];
        }
    }
    
    if ( ports.count > 0 && _messageLabel ) {
        [_messageLabel removeFromSuperview];
    } else if ( ports.count == 0 && !_messageLabel ) {
        self.messageLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _messageLabel.translatesAutoresizingMaskIntoConstraints = NO;
        _messageLabel.backgroundColor = [UIColor clearColor];
        _messageLabel.text = @"Please connect to Audiobus";
        _messageLabel.textColor = [UIColor whiteColor];
        [_scrollView addSubview:_messageLabel];
        [_customConstraints addObject:[NSLayoutConstraint constraintWithItem:_messageLabel attribute:NSLayoutAttributeCenterX relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeCenterX multiplier:1 constant:0]];
        [_customConstraints addObject:[NSLayoutConstraint constraintWithItem:_messageLabel attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:self.view attribute:NSLayoutAttributeCenterY multiplier:1 constant:0]];
    }
    
    [self.view addConstraints:_customConstraints];
    
    [self.view layoutIfNeeded];
    
}

@end
