//
//  ABReceiverAppDelegate.m
//  Audiobus Samples
//
//  Created by Michael Tyson on 15/12/2013.
//  Copyright (c) 2011-2014 Audiobus. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
//
//  - Redistributions in binary form must reproduce the above copyright notice, this
//    list of conditions and the following disclaimer in the documentation and/or
//    other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "ABReceiverAppDelegate.h"
#import "ABReceiverViewController.h"
#import "Audiobus.h"
#import "ABReceiverAudioEngine.h"

static void * kPlayingChanged = &kPlayingChanged;
static void * kRecordingChanged = &kRecordingChanged;

@interface ABReceiverAppDelegate () <ABAudiobusControllerStateIODelegate> {
    AudioUnit _audioUnit;
    
}
@property (nonatomic, strong) ABReceiverViewController *viewController;
@property (nonatomic, strong, readwrite) ABReceiverAudioEngine *audioEngine;
@property (nonatomic, strong) ABTrigger *playTrigger;
@property (nonatomic, strong) ABTrigger *recordTrigger;
@end

@implementation ABReceiverAppDelegate

- (void)dealloc {
    [_viewController removeObserver:self forKeyPath:@"playing"];
    [_viewController removeObserver:self forKeyPath:@"recording"];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    
    application.statusBarHidden = YES;
    
    [[NSUserDefaults standardUserDefaults] registerDefaults:@{ @"Monitor Volume": @(1.0) }];
    
    self.audioEngine = [ABReceiverAudioEngine new];
    _audioEngine.monitorVolume = [[NSUserDefaults standardUserDefaults] doubleForKey:@"Monitor Volume"];
    [_audioEngine start];
    
    // Set ourselves as the State IO delegate, so we can send and receive app state when the user saves/loads a preset
    _audioEngine.audiobusController.stateIODelegate = self;
    
    // Add triggers for play and record
    self.recordTrigger = [ABTrigger triggerWithSystemType:ABTriggerTypeRecordToggle
                                                    block:^(ABTrigger *trigger, NSSet *ports) { [_viewController toggleRecord:nil]; }];
    self.playTrigger = [ABTrigger triggerWithSystemType:ABTriggerTypePlayToggle
                                                  block:^(ABTrigger *trigger, NSSet *ports) { [_viewController togglePlay:nil]; }];
    [_audioEngine.audiobusController addTrigger:_recordTrigger];
    [_audioEngine.audiobusController addTrigger:_playTrigger];
    
    self.viewController = [[ABReceiverViewController alloc] initWithAudioEngine:_audioEngine];
    self.window.rootViewController = self.viewController;
    [self.window makeKeyAndVisible];
    
    [_audioEngine addObserver:self forKeyPath:@"playing" options:0 context:kPlayingChanged];
    [_audioEngine addObserver:self forKeyPath:@"recording" options:0 context:kRecordingChanged];
    
    return YES;
}

-(NSDictionary *)audiobusStateDictionaryForCurrentState {
    // This method is called by Audiobus when the user saves a preset. This lets the app set some state to be
    // restored when the preset is loaded again. In this case we'll save the monitor volume.
    return @{ @"Monitor Volume": @(_audioEngine.monitorVolume),
              @"Monitoring Enabled": @(_audioEngine.monitoringEnabled) };
}

-(void)loadStateFromAudiobusStateDictionary:(NSDictionary *)dictionary responseMessage:(NSString *__autoreleasing *)outResponseMessage {
    // This will be called when the user loads a preset in Audiobus. Passed in is the same dictionary that was
    // returned above when the preset was saved, so we can restore state from it.
    _audioEngine.monitorVolume = [dictionary[@"Monitor Volume"] floatValue];
    _audioEngine.monitoringEnabled = [dictionary[@"Monitoring Enabled"] boolValue];
}

-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    if ( context == kPlayingChanged ) {
        _playTrigger.state = _audioEngine.playing ? ABTriggerStateSelected : ABTriggerStateNormal;
    } else if ( context == kRecordingChanged ) {
        _recordTrigger.state = _audioEngine.recording ? ABTriggerStateSelected : ABTriggerStateNormal;
    } else {
        [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
}

@end
