//
//  ABKeyButton.m
//  Audiobus Samples
//
//  Created by Michael Tyson on 17/12/2013.
//  Copyright (c) 2011-2014 Audiobus. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
//
//  - Redistributions in binary form must reproduce the above copyright notice, this
//    list of conditions and the following disclaimer in the documentation and/or
//    other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "ABKeyButton.h"
#import <QuartzCore/QuartzCore.h>

@implementation ABKeyButton

+ (ABKeyButton*)keyButtonWithType:(ABKeyButtonType)type {
    BOOL isiPad = [[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad;
    
    ABKeyButton *button = [[ABKeyButton alloc] initWithFrame:CGRectZero];
    button.type = type;
    button.layer.cornerRadius = isiPad ? 10.0 : 7.0;
    
    if ( type == ABKeyButtonTypeBlack ) {
        button.backgroundColor = [UIColor colorWithRed:0.294 green:0.561 blue:0.651 alpha:1.000];
    } else {
        button.backgroundColor = [UIColor whiteColor];
        button.layer.borderColor = [UIColor colorWithRed:0.294 green:0.561 blue:0.651 alpha:1.000].CGColor;
        button.layer.borderWidth = 2.0;
    }
    
    return button;
}

-(void)setSelected:(BOOL)selected {
    [super setSelected:selected];
    [self updateColor];
}

-(void)setHighlighted:(BOOL)highlighted {
    [super setHighlighted:highlighted];
    [self updateColor];
}

-(void)updateColor {
    if ( _type == ABKeyButtonTypeBlack ) {
        self.backgroundColor = self.selected || self.highlighted ? [UIColor colorWithRed:0.115 green:0.259 blue:0.348 alpha:1.000] : [UIColor colorWithRed:0.294 green:0.561 blue:0.651 alpha:1.000];
    } else {
        self.backgroundColor = self.selected || self.highlighted ? [UIColor colorWithWhite:1.0 alpha:0.8] : [UIColor whiteColor];
    }
}

@end
