//
//  ABSenderAppDelegate.m
//  Audiobus Samples
//
//  Created by Michael Tyson on 17/12/2013.
//  Copyright (c) 2011-2014 Audiobus. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//
//  - Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
//
//  - Redistributions in binary form must reproduce the above copyright notice, this
//    list of conditions and the following disclaimer in the documentation and/or
//    other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "ABSenderAppDelegate.h"
#import "ABSenderViewController.h"
#import "ABSenderAudioEngine.h"
#import "Audiobus.h"

@interface ABSenderAppDelegate () <UIApplicationDelegate, ABAudiobusControllerStateIODelegate>
@property (strong, nonatomic, readwrite) ABSenderAudioEngine *audioEngine;
@property (strong, nonatomic) ABSenderViewController *viewController;
@end

@implementation ABSenderAppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    
    [[NSUserDefaults standardUserDefaults] registerDefaults:@{ @"Waveform": @(ABSenderWaveformSine), @"Attack": @(0.002), @"Decay": @(0.1) }];
    
    application.statusBarHidden = YES;
    
    self.audioEngine = [ABSenderAudioEngine new];
    [_audioEngine start];
    _audioEngine.audiobusController.stateIODelegate = self;
    
    [self setupRemoteTriggerMatrix];
    
    NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
    _audioEngine.waveform = (ABSenderWaveform)[ud integerForKey:@"Waveform"];
    _audioEngine.attack = [ud doubleForKey:@"Attack"];
    _audioEngine.decay = [ud doubleForKey:@"Decay"];
    
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.viewController = [[ABSenderViewController alloc] initWithAudioEngine:_audioEngine];
    self.window.rootViewController = self.viewController;
    [self.window makeKeyAndVisible];
    
    return YES;
}

-(NSDictionary *)audiobusStateDictionaryForCurrentState {
    // This method is called by Audiobus when the user saves a preset. This lets the app set some state to be
    // restored when the preset is loaded again.
    return @{ @"Waveform": @(_audioEngine.waveform),
              @"Attack": @(_audioEngine.attack),
              @"Decay": @(_audioEngine.decay) };
}

-(void)loadStateFromAudiobusStateDictionary:(NSDictionary *)dictionary responseMessage:(NSString *__autoreleasing *)outResponseMessage {
    // This will be called when the user loads a preset in Audiobus. Passed in is the same dictionary that was
    // returned above when the preset was saved, so we can restore state from it.
    _audioEngine.waveform = (ABSenderWaveform)[dictionary[@"Waveform"] integerValue];
    _audioEngine.attack = [dictionary[@"Attack"] doubleValue];
    _audioEngine.decay = [dictionary[@"Decay"] doubleValue];
}

- (void) setupRemoteTriggerMatrix {

    int numberOfRows = 4;
    int numberOfCols = 3;
    int semitone = 0;
    NSMutableArray *remoteMatrixTriggers = [NSMutableArray array];
    
    // Create a number of triggers, each to play a different note
    for ( int row = 0; row < numberOfRows; row++ ) {
        for ( int col = 0; col < numberOfCols; col++ ) {
            
            // Create the trigger button
            UIImage *image = [UIImage imageNamed:[NSString stringWithFormat:@"TriggerIcon%@",@(semitone+1)]];
            ABButtonTrigger *buttonTrigger = [ABButtonTrigger buttonTriggerWithTitle:[NSString stringWithFormat:@"Note %d", semitone+1] icon:image block:nil];
            
            // Play note on touch down
            [buttonTrigger addBlock:^(ABTrigger *trigger,
                                      NSSet *ports) {
                [_audioEngine toneOn:semitone];
            } forRemoteControlEvents:UIControlEventTouchDown];
            
            // Release note on touch up
            [buttonTrigger addBlock:^(ABTrigger *trigger,
                                      NSSet *ports) {
                [_audioEngine toneOff:semitone];
            } forRemoteControlEvents:(UIControlEventTouchUpInside | UIControlEventTouchUpOutside)];
            
            [remoteMatrixTriggers addObject:buttonTrigger];
            
            semitone++;
        }
    }
    
    // Now add all of the triggers as a matrix
    [_audioEngine.audiobusController addRemoteTriggerMatrix:remoteMatrixTriggers
                                                       rows:numberOfRows
                                                       cols:numberOfCols];
}

@end
